/*
  This file sets up the game's global variables
*/

var STAGE_WIDTH = 600
  , STAGE_HEIGHT = 600
  , TILES_COUNT = 12
  , SPRITE_WIDTH = 40
  , SPRITE_HEIGHT = 40
  , TILE_SIZE = Math.floor(STAGE_WIDTH / TILES_COUNT);

var GG = {
  game : null,
  hero : null,
  hero_skin : 'game_hero',
  hero_speed : 5, // this actual player speed in runtime, Game.hero.speed is a gamedev setting

  walls : [],
  wall_skin : 'game_wall',

  intervals : []

};

var Item = {
  freeze : "freeze",
  speed : "speed",
  slow : "slow"
}

function displayDebug(message) {
  var debug_ui = document.getElementById('debug_ui');
  debug_ui.style.display = "block";
  var debug_message = document.getElementById('debug_message');
  debug_message.innerHTML = message;
}

function GGException(args, msg){
  var funcName = args.callee.toString();
  funcName = funcName.substr('function '.length);
  funcName = funcName.substr(0, funcName.indexOf('('));

  this.args = Array.prototype.slice.call(args);
  this.func = funcName;
  this.msg = msg;
  this.toString = function(){
    return "Game Dev Error in "+this.func+"() arguments: ["+this.args.toString()+"]\nError Message: "+this.msg;
  }

  // display error in debug_ui
  displayDebug(this.toString());
}

// assigns GG.hero = {}
function spawnHero(x,y){
  GG.hero = {x:x,y:y}
  return GG.hero;
}


function placeWall(x,y){
  var wall = {
    x : (x>-1)?x:false || Math.floor(Math.random()*TILES_COUNT),
    y : (y>-1)?y:false || Math.floor(Math.random()*TILES_COUNT)
  }
  if(!(x>-1)&&!(y>-1)) wall = rerollPosition("wall",wall);

  internal_spawnWall(wall);

  return wall;
}



var x = "x";
var y = "y";


// INTERNAL
function internal_spawnWall(_wall,init) {
  var wall = new Sprite(SPRITE_WIDTH,SPRITE_HEIGHT);
  wall.x = _wall.x * TILE_SIZE;
  wall.y = _wall.y * TILE_SIZE;

  // ##TODO check for absolute url
  wall.image = GG.game.assets['assets/'+GG.wall_skin+'.png'];

  //if(init === undefined)
  GG.walls.push(_wall);

  GG.game.rootScene.addChild(wall);

  GG.game.walls.push(wall);
}

function rerollPosition(type_thats_checking,obj){
  // check if obj is in a taken position
  var max_check = 0;
  while(!tileIsVacant(type_thats_checking,obj.x,obj.y) && max_check++ < 50){
    // console.log("tile isn't vacant, rerolling");
    obj.x = Math.floor(Math.random()*TILES_COUNT);
    obj.y = Math.floor(Math.random()*TILES_COUNT);
  }

  return obj;
}
function tileIsVacant(type_thats_checking,x,y) {

  // check if any walls or gems are here
  // console.log(GG.walls.length);
  if(type_thats_checking != "wall"){
    for (var i = 0; i < GG.walls.length; i++) {
      if(
        (GG.walls[i].x == x && GG.walls[i].y == y)
        || tileIsTouching(GG.game.walls[i],x,y) ){
        return false;
      }
    }
  }


  return true;
}
function tileIsTouching(obj,x,y) {
  var ent1 = new Entity();
  ent1.x      = x*TILE_SIZE;
  ent1.y      = y*TILE_SIZE;
  ent1.width  = SPRITE_WIDTH;
  ent1.height = SPRITE_HEIGHT;
  GG.game.rootScene.addChild(ent1);
  var touching = obj.intersect(ent1);
  GG.game.rootScene.removeChild(ent1);
  if(touching){
      console.log("tileIsTouching: "+touching);
  }
  return touching;
}
